/*
 *	Copyright 2011 Paul Parsons
 *
 *	This file is part of TinyInit.
 *
 *	TinyInit is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	TinyInit is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TinyInit.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>

/*
 *	Shared memory for Client <--> init / Daemon <--> Daemon.
 */

#define	INIT_KEY	0x74690000	/* "ti" version 0x0000 */

#define	SERVNAMELEN	32		/* Maximum length of service name */
#define	DMONNAMELEN	32		/* Maximum length of daemon name */

#define	MAX_DEPS	32		/* Maximum number of dependent services */
#define	MAX_DMON	80		/* Maximum number of daemons */

enum clientcmd
{
	CC_NULL=-1,	/* (null) */
	CC_DEPEND,	/* service --depend */
	CC_START,	/* service --start */
	CC_STOP,	/* service --stop */
	CC_LIST,	/* service --list */
	CC_HALT,	/* halt */
};

typedef struct { char name[SERVNAMELEN]; } servname_t;

struct tinydeps
{
	pid_t		ppid;		/* Program PID */
	int		count;		/* Service name count */
	int		pad[2];		/* (pad to 16-byte boundary) */
	servname_t	sn[MAX_DEPS];	/* Service names */
};

enum halttype
{
	HT_NULL=-1,	/* (null) */
	HT_HALT,
	HT_POWEROFF,
	HT_REBOOT,
	HT_count
};

typedef struct { char name[DMONNAMELEN]; } dmonname_t;

struct tinyshm
{
	/*
	 *	Client <--> init.
	 */
	unsigned	cseq;		/* Client sequence number */
	enum clientcmd	ccmd;		/* Client command */
	pid_t		cpid;		/* Client PID */
	int		pad;		/* (pad to 16-byte boundary) */
	union
	{
		struct tinydeps	depend;		/* CC_DEPEND */
		servname_t	sn;		/* CC_START, CC_STOP */
		key_t		key;		/* CC_LIST */
		enum halttype	type;		/* CC_HALT */
	} u;

	/*
	 *	Daemon <--> Daemon.
	 */
	pid_t		dpid[MAX_DMON];	/* Daemon PIDs */
	dmonname_t	dn[MAX_DMON];	/* Daemon names */
};

/*
 *	Signals for Client <--> init.
 */

#define	SIGPOKE		SIGUSR1		/* Client -> init */
#define	SIGACK		SIGUSR1		/* Client <- init */
#define	SIGNAK		SIGUSR2		/* Client <- init */

/*
 *	Service table entry.
 */

#define	MAX_SERVICES	64		/* MUST be multiple of sizeof(int)*8 */

#define	BIT_TO_IDX(bit)	((bit) / (sizeof(int)*8))
#define	IDX_TO_BIT(idx)	((idx) * (sizeof(int)*8))
#define	BITMAPBANKS	(BIT_TO_IDX(MAX_SERVICES))

enum servstate
{
	SS_STOPPED,	/* Service stopped; Program stopped; Table entry free */
	SS_STARTING,	/* Service stopped; Program running */
	SS_BLOCKED,	/* Service stopped; Program blocked */

	SS_STARTED,	/* Service started; Program stopped */
	SS_STOPPING,	/* Service started; Program running */

	SS_count
};

typedef unsigned bitmap_t[BITMAPBANKS];

struct servtab
{
	servname_t	sn;		/* Service name */
	enum servstate	state;		/* Service state */
	unsigned	refcnt;		/* Current number of parent services */
	pid_t		ppid;		/* Service program PID */
	pid_t		spid;		/* /sbin/service PID */
	bitmap_t	bmap;		/* Dependent services bitmap */
};

/*
 *	Global declarations.
 */

/* killkill.c */
extern int kill_kill(pid_t pid, int sig);

/* killwait.c */
extern int kill_wait(pid_t pid, int sig);

/* reboot.c */
extern void myreboot(enum halttype type, int nosync);

/* sem_dn.c */
extern int sem_dn(int semid);

/* sem_up.c */
extern int sem_up(int semid);

/* sstdio.c */
extern int set_stdio(const char *dev);
