/*
 *	Copyright 2011 Paul Parsons
 *
 *	This file is part of TinyInit.
 *
 *	TinyInit is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	TinyInit is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TinyInit.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <signal.h>
#include <time.h>

#include <sys/wait.h>

#include "init.h"

/*
 *	Kill a child process, pid, with sig.
 *	Check for the existence of pid at 1/10s intervals.
 *	Kill pid with SIGKILL at 1 second.
 *	Check for the existence of pid at 1/10s intervals.
 *	Fail at 2 seconds.
 */

#define	FRAC	10		/* Fractions of a second */

int kill_wait(pid_t pid, int sig)
{
	struct timespec rqt;
	int i;

	rqt.tv_sec = 0;
	rqt.tv_nsec = (1000000000L/10);	/* 1/10 second */

	i = 0;
again:
	if (kill(pid, sig) == -1 && errno == ESRCH)
		return (0);
	do
	{
		(void) nanosleep(&rqt, NULL);
		if (waitpid(pid, NULL, WNOHANG) != 0)
		{
			/* ((wait -> -1 && errno == ECHILD) || wait -> pid) */
			return (0);
		}
		++i;
	} while ((i % FRAC) != 0);
	if (i == (FRAC*2))
		return (-1);
	sig = SIGKILL;
	goto again;
}
