/*
 *	Copyright 2011 Paul Parsons
 *
 *	This file is part of TinyInit.
 *
 *	TinyInit is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	TinyInit is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TinyInit.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>

#if	defined(__gnu_linux__)
#include <sys/reboot.h>
#elif	defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
#include <unistd.h>
#include <sys/reboot.h>
#else
#error	"Unknown operating system"
#endif

#include "init.h"

static const int cmd[HT_count] =
{
#if	defined(__gnu_linux__)
	[HT_HALT]=	RB_HALT_SYSTEM,
	[HT_POWEROFF]=	RB_POWER_OFF,
	[HT_REBOOT]=	RB_AUTOBOOT,
#elif	defined(__FreeBSD__)
	[HT_HALT]=	RB_HALT,
	[HT_POWEROFF]=	RB_POWEROFF,
	[HT_REBOOT]=	RB_AUTOBOOT,
#elif	defined(__NetBSD__)
	[HT_HALT]=	RB_HALT,
	[HT_POWEROFF]=	RB_POWERDOWN,
	[HT_REBOOT]=	RB_AUTOBOOT,
#elif	defined(__OpenBSD__)
	[HT_HALT]=	RB_HALT,
	[HT_POWEROFF]=	RB_HALT|RB_POWERDOWN,
	[HT_REBOOT]=	RB_AUTOBOOT,
#else
#error	"Unknown operating system"
#endif
};

void myreboot(enum halttype type, int nosync)
{
#if	defined(__gnu_linux__)
	if (! nosync)
		sync();
	(void) reboot(cmd[type]);
#elif	defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
	int howto;
	howto = cmd[type];
	if (nosync)
		howto |= RB_NOSYNC;
#if	defined(__FreeBSD__) || defined(__OpenBSD__)
	(void) reboot(howto);
#else
	(void) reboot(howto, NULL);
#endif
#else
#error	"Unknown operating system"
#endif
}
